@@ -35,14 +35,36 @@ use AnyEvent (); BEGIN { AnyEvent::common_sense }
 use AnyEvent::Util qw(AF_INET6);
 
 our $VERSION = $AnyEvent::VERSION;
+our @DNS_FALLBACK; # some public dns servers as fallback
 
-# some public dns servers
-our @DNS_FALLBACK = (
-   (v8.8.8.8, v8.8.4.4)[rand 2], # google public dns
-   (v209.244.0.3, v209.244.0.4)[rand 2], # level3
-   (v4.2.2.1, v4.2.2.3, v4.2.2.4, v4.2.2.5, v4.2.2.6)[rand 4], # vnsc-pri.sys.gtei.net
-);
-push @DNS_FALLBACK, splice @DNS_FALLBACK, rand $_, 1 for reverse 1..@DNS_FALLBACK;
+{
+   my $prep = sub {
+      $_ = $_->[rand @$_] for @_;
+      push @_, splice @_, rand $_, 1 for reverse 1..@_; # shuffle
+      $_ = pack "H*", $_ for @_;
+      \@_
+   };
+
+   my $ipv4 = $prep->(
+      ["08080808", "08080404"], # 8.8.8.8, 8.8.4.4 - google public dns
+#      ["d1f40003", "d1f30004"], # v209.244.0.3/4 - resolver1/2.level3.net - status unknown
+      ["04020201", "04020203", "04020204", "04020205", "04020206"], # v4.2.2.1/3/4/5/6 - vnsc-pri.sys.gtei.net - effectively public
+      ["cdd22ad2", "4044c8c8"], # 205.210.42.205, 64.68.200.200 - cache1/2.dnsresolvers.com - verified public
+#      ["8d010101"], # 141.1.1.1 - cable&wireless - status unknown
+   );
+
+   my $ipv6 = $prep->(
+      ["20014860486000000000000000008888", "20014860486000000000000000008844"], # 2001:4860:4860::8888/8844 - google ipv6
+   );
+
+   undef $ipv4 unless $AnyEvent::PROTOCOL{ipv4};
+   undef $ipv6 unless $AnyEvent::PROTOCOL{ipv6};
+
+   ($ipv6, $ipv4) = ($ipv4, $ipv6)
+      if $AnyEvent::PROTOCOL{ipv6} > $AnyEvent::PROTOCOL{ipv4};
+
+   @DNS_FALLBACK = (@$ipv4, @$ipv6);
+}
 
 =item AnyEvent::DNS::a $domain, $cb->(@addrs)
 
@@ -704,7 +726,7 @@ sub resolver() {
       ;
 
       $ENV{PERL_ANYEVENT_RESOLV_CONF}
-         ? $RESOLVER->_parse_resolv_conf_file ($ENV{PERL_ANYEVENT_RESOLV_CONF})
+         ? $RESOLVER->_load_resolv_conf_file ($ENV{PERL_ANYEVENT_RESOLV_CONF})
          : $RESOLVER->os_config;
 
       $RESOLVER
@@ -721,9 +743,9 @@ The following options are supported:
 
 =item server => [...]
 
-A list of server addresses (default: C<v127.0.0.1>) in network format
-(i.e. as returned by C<AnyEvent::Socket::parse_address> - both IPv4 and
-IPv6 are supported).
+A list of server addresses (default: C<v127.0.0.1> or C<::1>) in network
+format (i.e. as returned by C<AnyEvent::Socket::parse_address> - both IPv4
+and IPv6 are supported).
 
 =item timeout => [...]
 
@@ -774,6 +796,7 @@ sub new {
       max_outstanding => 10,
       reuse   => 300,
       %arg,
+      inhibit => 0,
       reuse_q => [],
    }, $class;
 
@@ -818,6 +841,18 @@ sub new {
    $self
 }
 
+# called to start asynchronous configuration
+sub _config_begin {
+   ++$_[0]{inhibit};
+}
+
+# called when done with async config
+sub _config_done {
+   --$_[0]{inhibit};
+   $_[0]->_compile;
+   $_[0]->_scheduler;
+}
+
 =item $resolver->parse_resolv_conf ($string)
 
 Parses the given string as if it were a F<resolv.conf> file. The following
@@ -877,14 +912,21 @@ sub parse_resolv_conf {
    $self->_compile;
 }
 
-sub _parse_resolv_conf_file {
+sub _load_resolv_conf_file {
    my ($self, $resolv_conf) = @_;
 
-   open my $fh, "<", $resolv_conf
-      or Carp::croak "$resolv_conf: $!";
+   $self->_config_begin;
 
-   local $/;
-   $self->parse_resolv_conf (<$fh>);
+   require AnyEvent::IO;
+   AnyEvent::IO::aio_load ($resolv_conf, sub {
+      if (my ($contents) = @_) {
+         $self->parse_resolv_conf ($contents);
+      } else {
+         AE::log 4 => "$resolv_conf: $!";
+      }
+
+      $self->_config_done;
+   });
 }
 
 =item $resolver->os_config
@@ -893,15 +935,22 @@ Tries so load and parse F</etc/resolv.conf> on portable operating
 systems. Tries various egregious hacks on windows to force the DNS servers
 and searchlist out of the system.
 
+This method must be called at most once before trying to resolve anything.
+
 =cut
 
 sub os_config {
    my ($self) = @_;
 
+   $self->_config_begin;
+
    $self->{server} = [];
    $self->{search} = [];
 
    if ((AnyEvent::WIN32 || $^O =~ /cygwin/i)) {
+      # TODO: this blocks the program, but should not, but I
+      # am too lazy to implement and test it. need to boot windows. ugh.
+
       #no strict 'refs';
 
       # there are many options to find the current nameservers etc. on windows
@@ -932,7 +981,7 @@ sub os_config {
          1
       }) {
          # we use ipconfig parsing because, despite all its brokenness,
-         # it seems most stable in practise.
+         # it seems quite stable in practise.
          # unfortunately it wants a console window.
          # for good measure, we append a fallback nameserver to our list.
 
@@ -963,22 +1012,27 @@ sub os_config {
          }
       }
 
-      # always add the fallback servers
+      # always add the fallback servers on windows
       push @{ $self->{server} }, @DNS_FALLBACK;
 
-      $self->_compile;
+      $self->_config_done;
    } else {
-      # try resolv.conf everywhere else
+      # try /etc/resolv.conf everywhere else
 
-      $self->_parse_resolv_conf_file ("/etc/resolv.conf")
-         if -e "/etc/resolv.conf";
+      require AnyEvent::IO;
+      AnyEvent::IO::aio_stat ("/etc/resolv.conf", sub {
+         $self->_load_resolv_conf_file ("/etc/resolv.conf")
+            if @_;
+         $self->_config_done;
+      });
    }
 }
 
 =item $resolver->timeout ($timeout, ...)
 
-Sets the timeout values. See the C<timeout> constructor argument (and note
-that this method uses the values itself, not an array-reference).
+Sets the timeout values. See the C<timeout> constructor argument (and
+note that this method expects the timeout values themselves, not an
+array-reference).
 
 =cut
 
@@ -1000,7 +1054,7 @@ sub max_outstanding {
    my ($self, $max) = @_;
 
    $self->{max_outstanding} = $max;
-   $self->_scheduler;
+   $self->_compile;
 }
 
 sub _compile {
@@ -1010,8 +1064,10 @@ sub _compile {
    my %server; $self->{server} = [grep 0 < length, grep !$server{$_}++, @{ $self->{server} }];
 
    unless (@{ $self->{server} }) {
-      # use 127.0.0.1 by default, and one opendns nameserver as fallback
-      $self->{server} = [v127.0.0.1, $DNS_FALLBACK[rand @DNS_FALLBACK]];
+      # use 127.0.0.1/::1 by default, add public nameservers as fallback
+      my $default = $AnyEvent::PROTOCOL{ipv6} > $AnyEvent::PROTOCOL{ipv4}
+                    ? "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\1" : "\x7f\x00\x00\x01";
+      $self->{server} = [$default, @DNS_FALLBACK];
    }
 
    my @retry;
@@ -1050,7 +1106,7 @@ sub _recv {
 
    my ($port, $host) = AnyEvent::Socket::unpack_sockaddr ($peer);
 
-   return unless $port == 53 && grep $_ eq $host, @{ $self->{server} };
+   return unless $port == DOMAIN_PORT && grep $_ eq $host, @{ $self->{server} };
 
    $self->_feed ($pkt);
 }
@@ -1148,6 +1204,8 @@ sub _exec {
 sub _scheduler {
    my ($self) = @_;
 
+   return if $self->{inhibit};
+
    #no strict 'refs';
 
    $NOW = time;
@@ -1181,7 +1239,7 @@ sub _scheduler {
          $self->_exec ($req);
 
       } elsif (my $cb = shift @{ $self->{wait} }) {
-         # found a wait_for_slot callback, call that one first
+         # found a wait_for_slot callback
          $cb->($self);
 
       } else {
@@ -1328,74 +1386,78 @@ sub resolve($%) {
    my $cb = pop;
    my ($self, $qname, $qtype, %opt) = @_;
 
-   my @search = $qname =~ s/\.$//
-      ? ""
-      : $opt{search}
-        ? @{ $opt{search} }
-        : ($qname =~ y/.//) >= $self->{ndots}
-          ? ("", @{ $self->{search} })
-          : (@{ $self->{search} }, "");
+   $self->wait_for_slot (sub {
+      my $self = shift;
 
-   my $class = $opt{class} || "in";
+      my @search = $qname =~ s/\.$//
+         ? ""
+         : $opt{search}
+           ? @{ $opt{search} }
+           : ($qname =~ y/.//) >= $self->{ndots}
+             ? ("", @{ $self->{search} })
+             : (@{ $self->{search} }, "");
 
-   my %atype = $opt{accept}
-      ? map +($_ => 1), @{ $opt{accept} }
-      : ($qtype => 1);
+      my $class = $opt{class} || "in";
 
-   # advance in searchlist
-   my ($do_search, $do_req);
-   
-   $do_search = sub {
-      @search
-         or (undef $do_search), (undef $do_req), return $cb->();
+      my %atype = $opt{accept}
+         ? map +($_ => 1), @{ $opt{accept} }
+         : ($qtype => 1);
 
-      (my $name = lc "$qname." . shift @search) =~ s/\.$//;
-      my $depth = 10;
+      # advance in searchlist
+      my ($do_search, $do_req);
+      
+      $do_search = sub {
+         @search
+            or (undef $do_search), (undef $do_req), return $cb->();
 
-      # advance in cname-chain
-      $do_req = sub {
-         $self->request ({
-            rd => 1,
-            qd => [[$name, $qtype, $class]],
-         }, sub {
-            my ($res) = @_
-               or return $do_search->();
+         (my $name = lc "$qname." . shift @search) =~ s/\.$//;
+         my $depth = 10;
 
-            my $cname;
+         # advance in cname-chain
+         $do_req = sub {
+            $self->request ({
+               rd => 1,
+               qd => [[$name, $qtype, $class]],
+            }, sub {
+               my ($res) = @_
+                  or return $do_search->();
 
-            while () {
-               # results found?
-               my @rr = grep $name eq lc $_->[0] && ($atype{"*"} || $atype{$_->[1]}), @{ $res->{an} };
+               my $cname;
 
-               (undef $do_search), (undef $do_req), return $cb->(@rr)
-                  if @rr;
+               while () {
+                  # results found?
+                  my @rr = grep $name eq lc $_->[0] && ($atype{"*"} || $atype{$_->[1]}), @{ $res->{an} };
 
-               # see if there is a cname we can follow
-               my @rr = grep $name eq lc $_->[0] && $_->[1] eq "cname", @{ $res->{an} };
+                  (undef $do_search), (undef $do_req), return $cb->(@rr)
+                     if @rr;
 
-               if (@rr) {
-                  $depth--
-                     or return $do_search->(); # cname chain too long
+                  # see if there is a cname we can follow
+                  my @rr = grep $name eq lc $_->[0] && $_->[1] eq "cname", @{ $res->{an} };
 
-                  $cname = 1;
-                  $name = lc $rr[0][4];
+                  if (@rr) {
+                     $depth--
+                        or return $do_search->(); # cname chain too long
 
-               } elsif ($cname) {
-                  # follow the cname
-                  return $do_req->();
+                     $cname = 1;
+                     $name = lc $rr[0][4];
 
-               } else {
-                  # no, not found anything
-                  return $do_search->();
-               }
-             }
-         });
+                  } elsif ($cname) {
+                     # follow the cname
+                     return $do_req->();
+
+                  } else {
+                     # no, not found anything
+                     return $do_search->();
+                  }
+                }
+            });
+         };
+
+         $do_req->();
       };
 
-      $do_req->();
-   };
-
-   $do_search->();
+      $do_search->();
+   });
 }
 
 =item $resolver->wait_for_slot ($cb->($resolver))
@@ -1431,14 +1493,13 @@ sub wait_for_slot {
 
 use AnyEvent::Socket (); # circular dependency, so do not import anything and do it at the end
 
-1;
-
 =back
 
 =head1 AUTHOR
 
    Marc Lehmann <schmorp@schmorp.de>
-   http://home.schmorp.de/
+   http://anyevent.schmorp.de
 
 =cut
 
+1
