@@ -1,3 +1,5 @@
+=encoding utf-8
+
 =head1 NAME
 
 AnyEvent::Util - various utility functions.
@@ -60,7 +62,7 @@ BEGIN {
 
    while (my ($k, $v) = each %ERR) {
       next if eval "Errno::$k ()";
-      AE::log 8 => "AnyEvent::Util: broken Errno module, adding Errno::$k.";
+      AE::log 8 => "Broken Errno module, adding Errno::$k.";
 
       eval "sub Errno::$k () { $v }";
       push @Errno::EXPORT_OK, $k;
@@ -77,7 +79,7 @@ cannot use select on.
 
 This function gives you a pipe that actually works even on the broken
 windows platform (by creating a pair of TCP sockets on windows, so do not
-expect any speed from that, and using C<pipe> everywhere else).
+expect any speed from that) and using C<pipe> everywhere else.
 
 See C<portable_socketpair>, below, for a bidirectional "pipe".
 
@@ -126,7 +128,6 @@ BEGIN {
 
             # vista has completely broken peername/sockname that return
             # fantasy ports. this combo seems to work, though.
-            #
             (Socket::unpack_sockaddr_in getpeername $r)[0]
             == (Socket::unpack_sockaddr_in getsockname $w)[0]
                or (($! = WSAEINVAL), next);
@@ -192,7 +193,8 @@ windows, it is abysmally slow, do not expect more than 5..20 forks/s on
 that sucky platform (note this uses perl's pseudo-threads, so avoid those
 like the plague).
 
-Example: poor man's async disk I/O (better use L<IO::AIO>).
+Example: poor man's async disk I/O (better use L<AnyEvent::IO> together
+with L<IO::AIO>).
 
    fork_call {
       open my $fh, "</etc/passwd"
@@ -224,7 +226,7 @@ my @fork_queue;
 sub _fork_schedule;
 sub _fork_schedule {
    require Storable unless $Storable::VERSION;
-   require POSIX    unless $Storable::VERSION;
+   require POSIX    unless $POSIX::VERSION;
 
    while ($forks < $MAX_FORKS) {
       my $job = shift @fork_queue
@@ -393,7 +395,7 @@ guard.
 BEGIN {
    if (!$ENV{PERL_ANYEVENT_AVOID_GUARD} && eval { require Guard; $Guard::VERSION >= 0.5 }) {
       *guard = \&Guard::guard;
-      AE::log 8 => "AnyEvent::Util: using Guard module to implement guards.";
+      AE::log 8 => "Using Guard module to implement guards.";
    } else {
       *AnyEvent::Util::guard::DESTROY = sub {
          local $@;
@@ -403,7 +405,7 @@ BEGIN {
             ${$_[0]}->();
          };
 
-         AE::log 4 => "runtime error in AnyEvent::guard callback: $@" if $@;
+         AE::log 4 => "Runtime error in AnyEvent::guard callback: $@" if $@;
       };
 
       *AnyEvent::Util::guard::cancel = sub ($) {
@@ -414,7 +416,7 @@ BEGIN {
          bless \(my $cb = shift), "AnyEvent::Util::guard"
       };
 
-      AE::log 8 => "AnyEvent::Util: using pure-perl guard implementation.";
+      AE::log 8 => "Using pure-perl guard implementation.";
    }
 }
 
@@ -606,6 +608,7 @@ sub run_cmd {
             $cv->begin;
 
             fcntl $pr, AnyEvent::F_SETFD, AnyEvent::FD_CLOEXEC;
+            fh_nonblocking $pr, 1;
             my $w; $w = AE::io $pr, 0,
                "SCALAR" eq ref $ob
                   ? sub {
@@ -650,6 +653,7 @@ sub run_cmd {
             }
 
             fcntl $pw, AnyEvent::F_SETFD, AnyEvent::FD_CLOEXEC;
+            fh_nonblocking $pw, 1;
             my $w; $w = AE::io $pw, 1, sub {
                my $len = syswrite $pw, $data;
 
@@ -800,7 +804,7 @@ sub idn_nameprep($;$) {
       # load the mapping data
       unless (defined $uts46_imap) {
          require Unicode::Normalize;
-         require "lib/AnyEvent/Util/uts46data.pl";
+         require "AnyEvent/Util/uts46data.pl";
       }
 
       # uts46 nameprep
@@ -867,7 +871,7 @@ sub idn_nameprep($;$) {
    if (/[^0-9a-z\-.]/) {
       # load the mapping data
       unless (defined $uts46_imap) {
-         require "lib/AnyEvent/Util/uts46data.pl";
+         require "AnyEvent/Util/uts46data.pl";
       }
 
       vec $uts46_valid, ord, 1
@@ -954,15 +958,14 @@ sub idn_to_unicode($) {
    defined $res ? $res : $_[0]
 }
 
-
-1;
-
 =back
 
 =head1 AUTHOR
 
  Marc Lehmann <schmorp@schmorp.de>
- http://home.schmorp.de/
+ http://anyevent.schmorp.de
 
 =cut
 
+1
+
